@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.interactive - useful stuff in the interactive session
 ;;;
-;;;   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,9 +34,9 @@
 #!no-fold-case
 
 (define-module gauche.interactive
-  (export apropos describe d read-eval-print-loop
+  (export apropos d read-eval-print-loop print-mode
           ;; autoloaded symbols follow
-          info info-page reload ed
+          info info-page info-search reload ed
           reload-modified-modules module-reload-rules reload-verbose)
   )
 (select-module gauche.interactive)
@@ -105,11 +105,10 @@
 ;;; Describe - describe object
 ;;;
 
-(define-method describe () (describe *1)) ; for convenience
+;; NB: The base methods (describe (obj <top>)) and
+;; (describe-slots (obj <top>)) are defined in src/libobj.scm
 
-(define-method describe (object) ; default
-  (describe-common object)
-  (describe-slots object))
+(define-method describe () (describe *1)) ; for convenience
 
 (define-method describe ((s <symbol>))
   (describe-common s)
@@ -124,7 +123,7 @@
 (define-method describe ((n <integer>))
   (describe-common n)
   (when (exact? n)
-    (format #t "  (#x~x" n)
+    (format #t "  (#x~,,'_,8:x" n)
     (when (<= 1000 n #e1e26) ; 10^26 is approx to 2^89
       (let loop ([nn n] [unit '(_ Ki Mi Gi Ti Pi Ei Zi Yi)])
         (cond [(null? unit)]
@@ -163,6 +162,8 @@
               (if has-optional?
                 (append spnames '_) ; this works even spnames is ()
                 spnames))))
+  (and-let1 dis ((with-module gauche.object generic-dispatcher-info) g)
+    (format #t "dispatcher:\n  ~s\n" dis))
   (values))
 
 (define-method describe ((p <procedure>))
@@ -174,22 +175,6 @@
 
 (define d describe)
 
-(define (describe-common obj)
-  (format #t "~s is an instance of class ~a\n" obj (class-name (class-of obj))))
-
-(define (describe-slots obj)
-  (let* ([class (class-of obj)]
-         [slots (class-slots class)])
-    (unless (null? slots)
-      (format #t "slots:\n")
-      (dolist [s (map slot-definition-name slots)]
-        (format #t "  ~10s: ~a\n" s
-                (if (slot-bound? obj s)
-                  (with-output-to-string
-                    (^[] (write-limited (slot-ref obj s) 60)))
-                  "#<unbound>"))))
-    (values)))
-
 ;;;
 ;;; Enhanced REPL
 ;;;
@@ -264,17 +249,19 @@
 ;; than the screen height.  Once we complete text.line-edit, we make
 ;; the feature available through command-line options of gosh.
 
-(define-values (%prompter %reader)
-  (receive (r rl skipper)
+(define-values (%prompter %reader %line-edit-ctx)
+  (receive (r rl skipper ctx)
       (if (sys-getenv "GAUCHE_READ_EDIT")
         (make-editable-reader (^[] (default-prompt-string "$")))
-        (values #f #f #f))
-    (if (and r rl skipper)
+        (values #f #f #f #f))
+    (if (and r rl skipper ctx)
       (values (^[] #f)
-              (make-repl-reader r rl skipper))
+              (make-repl-reader r rl skipper)
+              ctx)
       (values (^[] (display (default-prompt-string)) (flush))
               (make-repl-reader read read-line
-                                consume-trailing-whitespaces)))))
+                                consume-trailing-whitespaces)
+              #f))))
 
 ;; error printing will be handled by the original read-eval-print-loop
 (define (%evaluator expr env)
@@ -283,14 +270,41 @@
       (%set-history-expr! r)
       (apply values r))))
 
+;; <write-controls> used for the printer.
+(define-constant *default-controls*
+  (make-write-controls :length 50 :level 10 :width 79
+                       :pretty (not (sys-getenv "GAUCHE_REPL_NO_PPRINT"))))
+(define *controls* *default-controls*)
+
+(define (%printer . exprs)
+  (dolist [expr exprs]
+    (write expr *controls*)
+    (newline)))
+
+;; API
+(define print-mode
+  (case-lambda
+    [() *controls*]                     ; return the current controls
+    [(c)                                ; set controls directly
+     (let1 c (if (eq? c 'default)
+               *default-controls*
+               c)
+       (assume-type c <write-controls>)
+       (rlet1 old *controls*
+         (set! *controls* c)))]
+    [kvs
+     (rlet1 old *controls*
+       (set! *controls* (apply write-controls-copy *controls* kvs)))]))
+
 ;; This shadows gauche#read-eval-print-loop
 (define (read-eval-print-loop :optional (reader #f)
                                         (evaluator #f)
                                         (printer #f)
                                         (prompter #f))
-  (let ([reader (or reader %reader)]
+  (let ([reader    (or reader %reader)]
         [evaluator (or evaluator %evaluator)]
-        [prompter (or prompter %prompter)])
+        [printer   (or printer %printer)]
+        [prompter  (or prompter %prompter)])
     ((with-module gauche read-eval-print-loop)
      reader evaluator printer prompter)))
 
@@ -298,8 +312,36 @@
 ;;; Misc. setup
 ;;;
 
+;; EXPERIMENTAL: windows console code page support for text.line-edit
+;; NB: ces (character encoding scheme) conversion is not implemented.
+(cond-expand
+ [gauche.os.windows
+  (autoload os.windows sys-get-console-output-cp)
+  ;; check if we have a windows console.
+  (when (or (sys-isatty (standard-input-port))
+            (sys-isatty (standard-output-port))
+            (sys-isatty (standard-error-port)))
+    ;; wide character settings for text.line-edit
+    (if-let1 ctx %line-edit-ctx
+      (case (sys-get-console-output-cp)
+        [(65001)
+         (set! (~ ctx 'wide-char-disp-setting 'mode) 'Surrogate)
+         (set! (~ ctx 'wide-char-pos-setting  'mode) 'Surrogate)
+         (set! (~ ctx 'wide-char-disp-setting 'wide-char-width) 2)
+         (set! (~ ctx 'wide-char-pos-setting  'wide-char-width) 1)
+         (set! (~ ctx 'wide-char-disp-setting 'surrogate-char-width) 2)
+         (set! (~ ctx 'wide-char-pos-setting  'surrogate-char-width) 2)]
+        [else ; 932 etc.
+         (set! (~ ctx 'wide-char-disp-setting 'mode) 'Surrogate)
+         (set! (~ ctx 'wide-char-pos-setting  'mode) 'Surrogate)
+         (set! (~ ctx 'wide-char-disp-setting 'wide-char-width) 2)
+         (set! (~ ctx 'wide-char-pos-setting  'wide-char-width) 2)
+         (set! (~ ctx 'wide-char-disp-setting 'surrogate-char-width) 4)
+         (set! (~ ctx 'wide-char-pos-setting  'surrogate-char-width) 4)])))]
+ [else])
+
 ;; Autoload online info viewer
-(autoload gauche.interactive.info info info-page)
+(autoload gauche.interactive.info info info-page info-search)
 
 ;; Autoload module reloader
 (autoload gauche.reload reload reload-modified-modules
@@ -314,23 +356,6 @@
 ;; See (describe <symbol>) above
 (autoload gauche.modutil describe-symbol-bindings)
 
-;; Load user-specific default settings from ~/.gaucherc, if there's any.
-;; It is evaluated in the user module.
-(let ((dotfile (sys-normalize-pathname "~/.gaucherc" :expand #t)))
-  (when (sys-access dotfile F_OK)
-    (load dotfile :environment (find-module 'user))))
-
 ;; This might help first time users
 (define-in-module user help "Type ,help (comma and help) for help")
 
-;; If gosh is invoked with R7RS mode, import r7rs-small libraries
-;; into user module for the convenience.
-(when (global-variable-ref (find-module 'user) '*r7rs-mode* #f)
-  (eval '(import (scheme base) (scheme case-lambda) (scheme char)
-                 (scheme complex) (scheme cxr) (scheme eval)
-                 (scheme file) (scheme inexact) (scheme lazy)
-                 (scheme load) (scheme process-context) (scheme read)
-                 (scheme repl) (scheme time) (scheme write)
-                 (only (gauche base) *1 *1+ *2 *2+ *3 *3+ *e *history))
-        (find-module 'r7rs.user)))
-
